@@ -1,5 +1,5 @@
 """
-SQLObject 0.13.0
+SQLObject 0.14.0
 ----------------
 
 :author: Ian Bicking <ianb@colorstudy.com>
@@ -143,26 +143,18 @@ def findDependantColumns(name, klass):
             depends.append(col)
     return depends
 
-def _collectAttributes(cls, new_attrs, look_for_class, delete=True,
-                       set_name=False, sort=False):
-    """
-    Finds all attributes in `new_attrs` that are instances of
-    `look_for_class`.  Returns them as a list.  If `delete` is true
-    they are also removed from the `cls`.  If `set_name` is true, then
-    the ``.name`` attribute is set for any matching objects.  If
-    `sort` is true, then they will be sorted by ``obj.creationOrder``.
+def _collectAttributes(cls, new_attrs, look_for_class):
+    """Finds all attributes in `new_attrs` that are instances of
+    `look_for_class`. The ``.name`` attribute is set for any matching objects.
+    Returns them as a list.
+
     """
     result = []
     for attr, value in new_attrs.items():
         if isinstance(value, look_for_class):
+            value.name = attr
+            delattr(cls, attr)
             result.append(value)
-            if set_name:
-                value.name = attr
-            if delete:
-                delattr(cls, attr)
-    if sort:
-        result.sort(
-            lambda a, b: cmp(a.creationOrder, b.creationOrder))
     return result
 
 class CreateNewSQLObject:
@@ -209,13 +201,11 @@ class sqlmeta(object):
     columnDefinitions = {}
 
     # These are lists of the join and index objects:
-    joins = []
     indexes = []
     indexDefinitions = []
+    joins = []
     joinDefinitions = []
 
-    __metaclass__ = declarative.DeclarativeMeta
-
     # These attributes shouldn't be shared with superclasses:
     _unshared_attributes = ['table', 'columns', 'childName']
 
@@ -242,6 +232,11 @@ class sqlmeta(object):
     childClasses = {} # References to child classes, keyed by childName
     childName = None # Class name for inheritance child object creation
 
+    # Does the row require syncing?
+    dirty = False
+
+    __metaclass__ = declarative.DeclarativeMeta
+
     def __classinit__(cls, new_attrs):
         for attr in cls._unshared_attributes:
             if not new_attrs.has_key(attr):
@@ -734,12 +729,9 @@ class SQLObject(object):
 
         cls._SO_setupSqlmeta(new_attrs, is_base)
 
-        implicitColumns = _collectAttributes(
-            cls, new_attrs, col.Col, set_name=True, sort=True)
-        implicitJoins = _collectAttributes(
-            cls, new_attrs, joins.Join, set_name=True)
-        implicitIndexes = _collectAttributes(
-            cls, new_attrs, index.DatabaseIndex, set_name=True)
+        implicitColumns = _collectAttributes(cls, new_attrs, col.Col)
+        implicitJoins = _collectAttributes(cls, new_attrs, joins.Join)
+        implicitIndexes = _collectAttributes(cls, new_attrs, index.DatabaseIndex)
 
         if not is_base:
             cls._SO_cleanDeprecatedAttrs(new_attrs)
@@ -772,32 +764,43 @@ class SQLObject(object):
         if connection and ('_connection' not in cls.__dict__):
             cls.setConnection(connection)
 
+        sqlmeta = cls.sqlmeta
+
         # We have to check if there are columns in the inherited
         # _columns where the attribute has been set to None in this
         # class.  If so, then we need to remove that column from
         # _columns.
-        for key in cls.sqlmeta.columnDefinitions.keys():
+        for key in sqlmeta.columnDefinitions.keys():
             if (key in new_attrs
                 and new_attrs[key] is None):
-                del cls.sqlmeta.columnDefinitions[key]
+                del sqlmeta.columnDefinitions[key]
 
-        for column in cls.sqlmeta.columnDefinitions.values():
-            cls.sqlmeta.addColumn(column)
+        for column in sqlmeta.columnDefinitions.values():
+            sqlmeta.addColumn(column)
 
         for column in implicitColumns:
-            cls.sqlmeta.addColumn(column)
+            sqlmeta.addColumn(column)
 
         # Now the class is in an essentially OK-state, so we can
         # set up any magic attributes:
         declarative.setup_attributes(cls, new_attrs)
 
-        if cls.sqlmeta.fromDatabase:
-            cls.sqlmeta.addColumnsFromDatabase()
+        if sqlmeta.fromDatabase:
+            sqlmeta.addColumnsFromDatabase()
 
         for j in implicitJoins:
-            cls.sqlmeta.addJoin(j)
+            sqlmeta.addJoin(j)
         for i in implicitIndexes:
-            cls.sqlmeta.addIndex(i)
+            sqlmeta.addIndex(i)
+
+        order_getter = lambda o: o.creationOrder
+        sqlmeta.columnList.sort(key=order_getter)
+        sqlmeta.indexes.sort(key=order_getter)
+        sqlmeta.indexDefinitions.sort(key=order_getter)
+        # Joins cannot be sorted because addJoin created accessors
+        # that remember indexes.
+        #sqlmeta.joins.sort(key=order_getter)
+        sqlmeta.joinDefinitions.sort(key=order_getter)
 
         # We don't setup the properties until we're finished with the
         # batch adding of all the columns...
@@ -812,7 +815,7 @@ class SQLObject(object):
             cls.q = sqlbuilder.SQLObjectTable(cls)
             cls.j = sqlbuilder.SQLObjectTableWithJoins(cls)
 
-        classregistry.registry(cls.sqlmeta.registry).addClass(cls)
+        classregistry.registry(sqlmeta.registry).addClass(cls)
 
     # @classmethod
     def _SO_setupSqlmeta(cls, new_attrs, is_base):
@@ -895,7 +898,7 @@ class SQLObject(object):
                 cache.put(id, cls, val)
             finally:
                 cache.finishPut(cls)
-        elif selectResults and not val.dirty:
+        elif selectResults and not val.sqlmeta.dirty:
             val._SO_writeLock.acquire()
             try:
                 val._SO_selectInit(selectResults)
@@ -936,7 +939,7 @@ class SQLObject(object):
                 raise SQLObjectNotFound, "The object %s by the ID %s does not exist" % (self.__class__.__name__, self.id)
         self._SO_selectInit(selectResults)
         self._SO_createValues = {}
-        self.dirty = False
+        self.sqlmeta.dirty = False
 
     def _SO_loadValue(self, attrName):
         try:
@@ -990,7 +993,7 @@ class SQLObject(object):
                 values = [(self.sqlmeta.columns[v[0]].dbName, v[1])
                           for v in self._SO_createValues.items()]
                 self._connection._SO_update(self, values)
-            self.dirty = False
+            self.sqlmeta.dirty = False
             self._SO_createValues = {}
         finally:
             self._SO_writeLock.release()
@@ -1040,7 +1043,7 @@ class SQLObject(object):
         if to_python:
             value = to_python(dbValue, self._SO_validatorState)
         if self.sqlmeta._creating or self.sqlmeta.lazyUpdate:
-            self.dirty = True
+            self.sqlmeta.dirty = True
             self._SO_createValues[name] = dbValue
             setattr(self, instanceName(name), value)
             return
@@ -1099,7 +1102,7 @@ class SQLObject(object):
                 except AttributeError, e:
                     raise AttributeError, '%s (with attribute %r)' % (e, name)
 
-            self.dirty = True
+            self.sqlmeta.dirty = True
             return
 
         self._SO_writeLock.acquire()
@@ -1282,7 +1285,7 @@ class SQLObject(object):
         # Get rid of _SO_create*, we aren't creating anymore.
         # Doesn't have to be threadsafe because we're still in
         # new(), which doesn't need to be threadsafe.
-        self.dirty = False
+        self.sqlmeta.dirty = False
         if not self.sqlmeta.lazyUpdate:
             del self._SO_createValues
         else:
